--[[
--
--  EkInspect 5.01
--
--  Refer to index.html for documentation and complete change history.
--
--  Author: Dargen of Eternal Keggers, Terenas.
--          http://www.eternalkeggers.net
--
--  Addons: http://www.wowinterface.com/portal.php?uid=25131
--          http://my.curse.com/dargen/projects/
--
--]]

-- ------
-- Constants
-- ------
EkInspect_Version = "5.01";
EkInspect_Slash1 = "/ekinspect";
EkInspect_Slash2 = "/ekins";

-- ------
-- Variables saved.
-- ------
EkInspect_FirstRun = nil;  -- nil==First time the addon has been run, 0==Not the first time.
EkInspect_MovableWindow = 0;   -- 0==Window not movable, 1==Movable
EkInspect_ShowTooltips = 1;  -- 1==Show detailed tooltips, 0=Don't.
EkInspect_ShowAtCursor = 0;  -- 1==Show detailed tooltips by the mouse pointer, 0==Show at default location (lower right corner of screen).
EkInspect_UI_Method = 1;  -- Inspection method: 1==Player, 2==Target, 3==Party, 4==Raid
EkInspect_UI_Slots = {};  -- Holds slot settings (what to inspect for each type of who to inspect) [who_id][what_id], or [who_id]["all"].
EkInspect_UI_Adjust = {};  -- Holds 'Adjust item levels of lower quality gear' setting: [who_id] == 1 if adjust, 0 if not to adjust.
EkInspect_InspectList = {};  -- Hold inspection data
EkInspect_InspectedMax = 0;

-- ------
-- Variables not saved.
-- ------
EkInspect_Test = 0;
EkInspect_VarsLoaded = false;
EkInspect_ShowDetails = false;
EkInspect_SummaryList = {};  -- Inspection summary
EkInspect_DetailsList = {};  -- Inspection details
EkInspect_Refilter = false;
EkInspect_Summary_Average = 0;


local MULTIBYTE_CHAR = "([\192-\255]?%a?[\128-\191]*)";

local function strlen_utf8(str)
	-- Get length of string in characters. This may not equal the number of bytes in the string.
	return strlenutf8(str);
end

local function strupper_utf8(str)
	-- Convert string to upper case.

	-- If entire string is composed of 1-byte characters, then use normal function.
	if (strlen_utf8(str) == strlen(str)) then
		return strupper(str);
	end

	return string.gsub(str, MULTIBYTE_CHAR, string.upper);
end

local function strlower_utf8(str)
	-- Convert string to lower case.

	-- If entire string is composed of 1-byte characters, then use normal function.
	if (strlen_utf8(str) == strlen(str)) then
		return strlower(str);
	end

	return string.gsub(str, MULTIBYTE_CHAR, string.lower);
end

local function strsub_utf8(str, startChar, endChar)
	-- Parameters:
	-- 	str -- string.
	-- 	startChar -- starting character position
	--		-- this is not a byte position.
	--		-- if negative, then starting character is counted from right side of string.
	--		-- if zero, then starting character is 1.
	--		-- if greater than number of characters in string, then returns empty string.
	-- 	endChar -- ending character position
	--		-- if nil, then all characters after startChar are returned.
	--		-- if negative, then ending character is counted from right side of string.
	--		-- if less than startChar, then returns empty string.

	local numChars = strlen_utf8(str);
	local numBytes = strlen(str);

	-- If entire string is composed of 1-byte characters, then use normal function.
	if (numChars == numBytes) then
		return strsub(str, startChar, endChar);
	end

	local bytePos = 1;
	local charPos = 1;
	local startByte;

	-- Examine the startChar parameter.
	if (startChar == 0) then
		startChar = 1;
	elseif (startChar < 1) then
		-- Recalculate starting character position from right side of string.
		startChar = numChars + startChar + 1;
		if (startChar < 1) then
			startChar = 1;
		end
	end
	if (startChar > numChars) then
		return "";
	end

	-- Locate the first character
	while (charPos < startChar and bytePos <= numBytes) do
		local char = string.byte(str, bytePos);
		if (char >= 240) then
			bytePos = bytePos + 4;
		elseif (char >= 224) then
			bytePos = bytePos + 3;
		elseif (char >= 192) then
			bytePos = bytePos + 2;
		else 
			bytePos = bytePos + 1;
		end
		charPos = charPos + 1;
	end
	startByte = bytePos;

	-- Examine the endChar parameter.
	if (endChar == nil) then
		return strsub(str, startByte);
	elseif (endChar < 0) then
		-- Recalculate ending character position from right side of string.
		endChar = numChars + endChar + 1;
		if (endChar < 1) then
			endChar = 1;
		end
	end
	if (endChar < startChar) then
		return "";
	end

	-- Extract the requested number of characters.
	while (endChar >= startChar and bytePos <= #str) do
		local char = string.byte(str, bytePos);
		if (char > 240) then
			bytePos = bytePos + 4;
		elseif (char > 224) then
			bytePos = bytePos + 3;
		elseif (char > 192) then
			bytePos = bytePos + 2;
		else 
			bytePos = bytePos + 1;
		end
		endChar = endChar - 1;
	end
	return strsub(str, startByte, bytePos - 1);
end

local function strname_utf8(str)
	-- Capitalize the first character and uncapitalize the rest.

	-- If entire string is composed of 1-byte characters, then use normal function.
	if (strlen_utf8(str) == strlen(str)) then
		return strupper(strsub(str, 1, 1)) .. strlower(strsub(str, 2));
	end

	return strupper_utf8(strsub_utf8(str, 1, 1)) .. strlower_utf8(strsub_utf8(str, 2));
end

function EkInspect_GetNumRaidMembers()
	if (IsInRaid()) then
		return GetNumGroupMembers();
	else
		return 0;
	end
end

function EkInspect_GetNumPartyMembers()
	return GetNumSubgroupMembers();
end

local GetNumRaidMembers = EkInspect_GetNumRaidMembers;
local GetNumPartyMembers = EkInspect_GetNumPartyMembers;


function EkInspect_OnLoad(self)
	-- --------------
	-- Initialize when addon is loaded.
	-- --------------
	SLASH_EKINSPECT1 = EkInspect_Slash1;
	SLASH_EKINSPECT2 = EkInspect_Slash2;

	SlashCmdList["EKINSPECT"] = function( msg )
		EkInspect_Command(msg);
	end

	self:RegisterEvent("PLAYER_LOGIN");
	self:RegisterEvent("GUILD_ROSTER_UPDATE");
end


function EkInspect_ShowVersion(print2)
	-- --------------
	-- Show version number of addon.
	-- --------------
	local text = "EkInspect " .. EkInspect_Version .. " by Dargen, Eternal Keggers, Terenas."
	if (print2) then
		EkInspect_Print2(text);
	else
		EkInspect_Print(text);
	end
end


function EkInspect_ShowInvalid()
	-- --------------
	-- Show help for invalid command.
	-- --------------
	EkInspect_Print(string.format(EkInspect_TEXT_Msg_Invalid_Command, EkInspect_Slash1));
end


function EkInspect_OnEvent(self, event, ...)
	-- --------------
	-- Envent handler.
	-- --------------
	if (event == "PLAYER_LOGIN") then
		if (not EkInspect_VarsLoaded) then
			EkInspect_VarsLoaded = true;
--			EkInspect_ShowVersion();

			for i = 1, #EkInspect_TEXT_Methods do
				if (not EkInspect_UI_Slots[i]) then
					EkInspect_UI_Slots[i] = {};
					-- Select all but the main hand, off hand items.
					for j, slotNames in ipairs(EkInspect_TEXT_SlotNames) do
						local id = slotNames.id;
						if (id == 17) then -- offhand slot
							EkInspect_UI_Slots[i][id] = nil;
--						elseif (id == 18) then -- ranged slot
--							EkInspect_UI_Slots[i][id] = nil;
						else
							EkInspect_UI_Slots[i][id] = 1;
						end
					end
				end
				if (not EkInspect_UI_Adjust[i]) then
					EkInspect_UI_Adjust[i] = 1;
				end
			end

			if (EkInspect_MovableWindow == 1) then
				EkInspect_Frame_UISpecialFrame(1);
				EkInspect_Frame_UIPanelSet(0);
				EkInspect_Frame_MoveButton:Show();
			else
				EkInspect_Frame_MoveButton:Hide();
				EkInspect_Frame_UISpecialFrame(0);
				EkInspect_Frame_UIPanelSet(1);
			end

			if (EkInspect_FirstRun == nil) then
				EkInspect_Print(EkInspect_TEXT_Msg_First_Run_Detected);
				EkInspect_ShowVersion();
				EkInspect_Print(string.format(EkInspect_TEXT_Msg_To_Open_Window, EkInspect_Slash1));
				EkInspect_FirstRun = 0;
			end

			EkInspect_SummaryFrame_ListInit();
			EkInspect_DetailsFrame_ListInit();

			EkInspect_Inspect_ApplyFilter();

			if (IsInGuild()) then
				GuildRoster();
			end
		end
		return;
	end

	if (not EkInspect_VarsLoaded) then
		return;
	end
end


function EkInspect_Command(msg)
	-- --------------
	-- Main command routine.
	-- --------------
	local pos1, pos2, command, options = string.find(msg, "^(%S*)%s*(.*)$");

	local lcommand = strlower(command);
	local loptions = strlower(options);

	local opts = EkInspect_ParseOptions(loptions);

	if (lcommand == nil or lcommand == "") then
		EkInspect_UIToggle();

	elseif (lcommand == nil or lcommand == "" or lcommand == EkInspect_TEXT_Command_Help or lcommand == "?") then
		-- Display help.
		EkInspect_Print2(" ");
		EkInspect_ShowVersion(2);
		EkInspect_Print2(" ");
		EkInspect_Print2(EkInspect_TEXT_Help_Summary);
		EkInspect_Print2(" ");
		EkInspect_Print2(EkInspect_Slash1 .. " " .. EkInspect_TEXT_Command_Player1 .. " -- " .. EkInspect_TEXT_Help_Command_Player1);
		EkInspect_Print2(EkInspect_Slash1 .. " " .. EkInspect_TEXT_Command_Player1 .. " " .. EkInspect_TEXT_Command_Option_Details .. " -- " .. EkInspect_TEXT_Help_Option_Player_Details);
		EkInspect_Print2(EkInspect_Slash1 .. " " .. EkInspect_TEXT_Command_Target .. " -- " .. EkInspect_TEXT_Help_Command_Target);
		EkInspect_Print2(EkInspect_Slash1 .. " " .. EkInspect_TEXT_Command_Target .. " " .. EkInspect_TEXT_Command_Option_Details .. " -- " .. EkInspect_TEXT_Help_Option_Target_Details);
		EkInspect_Print2(EkInspect_Slash1 .. " " .. EkInspect_TEXT_Command_Party .. " -- " .. EkInspect_TEXT_Help_Command_Party);
		EkInspect_Print2(EkInspect_Slash1 .. " " .. EkInspect_TEXT_Command_Raid .. " -- " .. EkInspect_TEXT_Help_Command_Raid);
		EkInspect_Print2(" ");
		EkInspect_Print2(EkInspect_Slash1 .. " " .. EkInspect_TEXT_Command_Help .. "/? -- " .. EkInspect_TEXT_Help_Command_Help);
		EkInspect_Print2(EkInspect_Slash1 .. " -- " .. EkInspect_TEXT_Help_Command);
		EkInspect_Print2(" ");
		EkInspect_Print2(EkInspect_TEXT_Help_Alternates .. " " .. EkInspect_Slash2);

	elseif (lcommand == EkInspect_TEXT_Command_Player1 or lcommand == EkInspect_TEXT_Command_Player2) then
		-- Inspect yourself
		EkInspect_Inspect_Perform(1, opts);
		EkInspect_PrintResults();

	elseif (lcommand == EkInspect_TEXT_Command_Target) then
		-- Inspect target
		EkInspect_Inspect_Perform(2, opts);
		EkInspect_PrintResults();

	elseif (lcommand == EkInspect_TEXT_Command_Party) then
		-- Inspect party
		EkInspect_Inspect_Perform(3, nil);
		EkInspect_PrintResults();

	elseif (lcommand == EkInspect_TEXT_Command_Raid) then
		-- Inspect raid
		EkInspect_Inspect_Perform(4, nil);
		EkInspect_PrintResults();

	elseif (lcommand == "_test") then
		EkInspect_TestMode();

	end
end

function EkInspect_PrintLine(msg)
	print(msg);
--	SendChatMessage(msg, "RAID");
--	SendChatMessage(msg, "PARTY");
--	SendChatMessage(msg, "WHISPER", nil, (UnitName("player")));
end

function EkInspect_PrintResults()
	-- ----------------
	-- Print inspection results.
	-- ----------------
	if (EkInspect_ShowDetails) then
		-- Sort by ascending item level.
		EkInspect_DetailsFrame_Ascend = 1;
		EkInspect_DetailsFrame_Sort("level");

		-- Output details
		for i, t in ipairs(EkInspect_DetailsList) do
			local playerTable = EkInspect_InspectList[t.name];
			local detailTable = playerTable.details[t.index];

			local slotName = "?";
			local id = detailTable.id;
			for i, slotNames in ipairs(EkInspect_TEXT_SlotNames) do
				if (slotNames.id == id) then
					slotName = slotNames.name;
					break;
				end
			end

			EkInspect_PrintLine(format("%s, iLevel=%d, %s.", slotName, detailTable.level, detailTable.link));
		end
		EkInspect_PrintLine(format("Average iLevel=%.2f.", EkInspect_Summary_Average));
	else
		-- Sort by ascending item level.
		EkInspect_SummaryFrame_Ascend = 1;
		EkInspect_SummaryFrame_Sort("level");

		-- Output summary
		for i, playerName in ipairs(EkInspect_SummaryList) do
			local playerTable = EkInspect_InspectList[playerName];
			EkInspect_PrintLine(format("%d. iLevel=%d, Avg=%.2f, %s.", i, playerTable.total, playerTable.average, playerName));
		end
		EkInspect_PrintLine(format("Average iLevel=%.2f, %d players out of %d.", EkInspect_Summary_Average, #EkInspect_SummaryList, EkInspect_InspectedMax));
	end
end

function EkInspect_TestMode()
	-- ----------------
	-- Toggle test mode
	-- ----------------
	if (EkInspect_Test == 1) then
		EkInspect_Print("Test mode disabled.");
		EkInspect_Test = 0;
		return;
	end
	-- Test mode will:
	-- - Do nothing at this time.
	EkInspect_Test = 1;
	EkInspect_Print("Test mode enabled.");
end

function EkInspect_Print(msg)
	-- ----------------
	-- Print a message.
	-- ----------------
	DEFAULT_CHAT_FRAME:AddMessage("<EkInspect> " .. msg);
	return;
end


function EkInspect_Print2(msg)
	-- ----------------
	-- Print a message.
	-- ----------------
	DEFAULT_CHAT_FRAME:AddMessage(msg);
	return;
end


function EkInspect_RaidSay(msg)
	-- ----------------
	-- Send a message to the raid channel.
	-- ----------------
	if (EkInspect_Test == 1) then
		EkInspect_Print(msg);
	else
		SendChatMessage("<EkInspect> " .. msg, "RAID");
	end
	return;
end


function EkInspect_SetTooltip_Cursor(owner, normalText)
	-- ----------
	-- Set tooltip anchored to the cursor.
	-- ----------
--	if (EkInspect_ShowTooltips ~= 1) then
--		return
--	end
	GameTooltip:SetOwner(owner, "ANCHOR_CURSOR");
	GameTooltip:SetText(normalText);
	GameTooltip:Show();
end

function EkInspect_SetTooltip_Anchor(owner, normalText, anchor, x, y, wrap)
	-- ----------
	-- Set tooltip using specified anchor.
	-- ----------
	local r, g, b, alpha;
--	if (EkInspect_ShowTooltips ~= 1) then
--		return
--	end
	GameTooltip:SetOwner(owner, anchor, x, y);
	GameTooltip:SetText(normalText, r, g, b, alpha, wrap);
	GameTooltip:Show();
end

function EkInspect_SetTooltip_Corner(owner, normalText, detailedText, anchor)
	-- ----------
	-- Set tooltip in lower right corner of screen.
	--
	-- Param: normalText -- Brief tooltip text.
	-- Param: detailedText -- Detailed tooltip text.
	-- ----------
	local r, g, b, alpha, wrap;
	if (EkInspect_ShowTooltips ~= 1) then
		return
	end
	r = 1;
	g = 1;
	b = 1;
	alpha = 1;
	wrap = 1;
	if (EkInspect_ShowAtCursor == 1) then
		if (not anchor) then
			local oType = owner:GetObjectType();
			if (oType == "CheckButton") then
				anchor = "ANCHOR_LEFT";
			elseif (oType == "Button") then
				anchor = "ANCHOR_BOTTOMRIGHT";
			elseif (oType == "EditBox") then
				anchor = "ANCHOR_RIGHT";
			elseif (oType == "Frame") then
				anchor = "ANCHOR_RIGHT";
			else
				anchor = "ANCHOR_RIGHT";
			end
		end
		GameTooltip:SetOwner(owner, anchor);
	else
		GameTooltip_SetDefaultAnchor(GameTooltip, owner);
	end
	if (normalText) then
		GameTooltip:SetText(normalText, r, g, b, alpha, wrap);
		if (detailedText) then
			GameTooltip:AddLine(detailedText, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1);
		end
	else
		GameTooltip:SetText(detailedText, r, g, b, alpha, wrap);
	end
	GameTooltip:Show();
end

function EkInspect_HideTooltip()
	-- ----------
	-- Hide the tooltip.
	-- ----------
	GameTooltip:Hide();
end

function EkInspect_SetButtonTextColor(button, r, g, b, a)
	-- ----------
	-- Set text color of specified button.
	-- ----------
	local text;
	text = button:GetText();
	if (strsub(text, 1, 2) == "|c") then
		text = strsub(text, 11);
	end
	if (not a) then
		a = 1;
	end
	button:SetText(string.format("|c%02x%02x%02x%02x%s", a * 255, r * 255, g * 255, b * 255, text));
end

function EkInspect_CanOfficerChat(rank)
	-- -----
	-- Test if player can speak in officer chat.
	-- (requires a previous GuildRoster() call at some point).
	--
	-- Note: This relies on the order of the flags returned
	--       by GuildControlGetRankFlags().
	--
	-- Param: rank -- 0==Guild master, 1==Officer, etc.
	-- -----
	if (IsInGuild()) then
		if (GuildControlGetNumRanks() == 0) then
			GuildRoster();
		end
		local guildName, rankName, rankIndex;
		if (rank) then
			rankIndex = rank;
		else
			-- Rank index returned by GetGuildInfo() starts at 0 for guild master.
			guildName, rankName, rankIndex = GetGuildInfo("player");
		end
		-- GuildControlSetRank() requires a rank 1 for guild master, so add 1.
		rankIndex = rankIndex + 1;
		if (rankIndex and rankIndex <= GuildControlGetNumRanks()) then
			GuildControlSetRank(rankIndex);
			local rankFlags = { GuildControlGetRankFlags() };
			-- Flag [4] is 1 if the rank can speak in officer chat.
		        if (rankFlags and rankFlags[4] == 1) then
				return true;
			end
		end
	end
	return false;
end

function EkInspect_ParseOptions(options)
	-- ----------------
	-- Parse command line options and return as an array.
	-- ----------------
	local strgmatch = string.gmatch or string.gfind;
	local optarray = {};

	for opt in strgmatch(options, "(%S+)") do
		table.insert(optarray, opt);
	end

	return optarray;
end

-- ------------------------------------------------------------------
-- Inspection routines
-- ------------------------------------------------------------------

function EkInspect_Inspect_Perform(method, opts)
	-- ----------------
	-- Perform the inspection.
	-- ----------------
	EkInspect_WhoWasInspected = method;
	EkInspect_InspectList = {};
	EkInspect_ShowDetails = false;

	if (opts) then
		for k, v in ipairs(opts) do
			if (v == "details" and (method == 1 or method == 2)) then
				EkInspect_ShowDetails = true;
			end
		end
	end

	EkInspect_InspectedMax = 0;

	if (method == 1) then
		EkInspect_Inspect_Unit("player");

	elseif (method == 2) then
		EkInspect_Inspect_Unit("target");

	elseif (method == 3) then
		for i = 1, GetNumPartyMembers() do
			EkInspect_Inspect_Unit("party" .. i);
		end
		EkInspect_Inspect_Unit("player");

	elseif (method == 4) then
		for i = 1, GetNumRaidMembers() do
			EkInspect_Inspect_Unit("raid" .. i);
		end
	end

	EkInspect_Inspect_ApplyFilter();
end

function EkInspect_Inspect_ApplyFilter()
	-- ----------------
	-- Apply filter to the inspect list.
	-- ----------------
	EkInspect_SummaryList = {};
	EkInspect_DetailsList = {};

	local slots = EkInspect_UI_Slots[EkInspect_WhoWasInspected];

	for playerName, playerTable in pairs(EkInspect_InspectList) do
		tinsert(EkInspect_SummaryList, playerName);

		for i, detailTable in ipairs(playerTable.details) do
			if (slots["all"] or slots[detailTable.id]) then
				tinsert(EkInspect_DetailsList, { name = playerName, index = i });
				detailTable.filter = 1;
			else
				detailTable.filter = nil;
			end
		end
	end

	EkInspect_Inspect_CalculateSummary();

	EkInspect_SummaryFrame_Selected = {};
	EkInspect_SummaryFrame_Sort(EkInspect_SummaryFrame_Order);
	EkInspect_SummaryFrame_Refresh();

	EkInspect_DetailsFrame_Selected = {};
	EkInspect_DetailsFrame_Sort(EkInspect_DetailsFrame_Order);
	EkInspect_DetailsFrame_Refresh();
end

function EkInspect_Inspect_Unit(unit)
	-- ----------------
	-- Inspect the specified unit.
	-- ----------------
	EkInspect_InspectedMax = EkInspect_InspectedMax + 1;

	if (not UnitExists(unit)) then
		return;
	end

	if (not CanInspect(unit, false)) then
		return;
	end

	local playerName = UnitName(unit);
	local playerTable = {};

	playerTable.class, playerTable.class2 = UnitClass(unit); -- localized, nonlocalized

	local detailsList = {};
	local count = 0;
	local total = 0;

	NotifyInspect(unit);

	local slots = EkInspect_UI_Slots[EkInspect_WhoWasInspected];

	for i, slotNames in ipairs(EkInspect_TEXT_SlotNames) do
		local id = slotNames.id;
		local slot = slotNames.slot;
		local link = GetInventoryItemLink(unit, slot);
		if (link) then
			local itemName, link, quality, itemLevel = GetItemInfo(link);

			-- Adjust item level based on quality of the item.
			local adjustedLevel = itemLevel;
			if (quality == 3) then  -- rare (blue)
				adjustedLevel = adjustedLevel - 13;
			elseif (quality == 2) then  -- uncommon (green)
				adjustedLevel = adjustedLevel - 26;
			elseif (quality == 1) then  -- common (white)
				adjustedLevel = adjustedLevel - 39;
			elseif (quality == 0) then  -- poor (grey)
				adjustedLevel = adjustedLevel - 52;
			end
			if (adjustedLevel < 0) then
				adjustedLevel = 0;
			end

			count = count + 1;
			total = total + itemLevel;

			tinsert(detailsList, { item = itemName, level = itemLevel, ilevel = itemLevel, adjusted = adjustedLevel, link = link, id = id, slot = slot, quality = quality });
		end
	end

	playerTable.details = detailsList;
	playerTable.total = total;
	playerTable.count = count;

	EkInspect_InspectList[playerName] = playerTable;
end



function EkInspect_Inspect_CalculateSummary()
	-- ----------------
	-- Calculate figures for the summary list.
	-- ----------------
	local gtotal = 0;
	local gcount = 0;
	local details;

	for i, playerName in ipairs(EkInspect_SummaryList) do
		local playerTable = EkInspect_InspectList[playerName];

		-- Add up the player's gear
		local total = 0;
		local count = 0;
		for j, detailTable in ipairs(playerTable.details) do
			if (detailTable.filter) then
				if (EkInspect_UI_Adjust[EkInspect_WhoWasInspected] == 1) then
					detailTable.level = detailTable.adjusted;
				else
					detailTable.level = detailTable.ilevel;
				end

				total = total + detailTable.level;
				count = count + 1;
				gcount = gcount + 1;
			end
		end

		-- Update the summary record for the player
		playerTable.total = total;

		if (count == 0) then
			playerTable.average = 0;
		else
			playerTable.average = total / count;
		end

		-- Accumulate the item level
		gtotal = gtotal + total;
	end

	-- Update value shown at bottom of summary list
	if (gcount == 0) then
		EkInspect_Summary_Average = 0;
	else
		EkInspect_Summary_Average = gtotal / gcount;
	end
end


-- ------------------------------------------------------------------
-- Show/hide/toggle/refresh UI panels
-- ------------------------------------------------------------------

function EkInspect_UIRefresh()
	-- ----
	-- Refresh the current UI frame.
	-- ----
	if (EkInspect_Frame_:IsShown()) then
		EkInspect_Frame_Refresh();
	end
end

function EkInspect_UIShow()
	-- Show the GUI
	if (not EkInspect_Frame_:IsShown()) then
		EkInspect_Frame_UIPanelShow(EkInspect_Frame_);
	end
end

function EkInspect_UIHide()
	-- Hide the GUI
	if (EkInspect_Frame_:IsShown()) then
		EkInspect_Frame_UIPanelHide(EkInspect_Frame_);
	end
end

function EkInspect_UIToggle()
	-- Toggle GUI visibility.
	if (EkInspect_Frame_:IsShown()) then
		EkInspect_Frame_UIPanelHide(EkInspect_Frame_);
	else
		EkInspect_Frame_UIPanelShow(EkInspect_Frame_);
	end
end


-- *!*
-- -------------------------------------------------------------------------
-- EkInspect_Frame
-- -------------------------------------------------------------------------

EkInspect_Frame_SUBFRAMES = {
	"EkInspect_MainFrame_",
	"EkInspect_SummaryFrame_",
	"EkInspect_DetailsFrame_",
	{"EkInspect_OptionsFrame_"},
};

EkInspect_Frame_TabMain = 1;
EkInspect_Frame_TabSummary = 2;
EkInspect_Frame_TabDetails = 3;
EkInspect_Frame_TabOptions = 4;

function EkInspect_Frame_UnlockWindow(movable)
	-- --------------
	-- Make window movable/unmovable.
	-- --------------
	EkInspect_MovableWindow = movable;
	EkInspect_Frame_UIPanelHide(EkInspect_Frame_);
	if ( movable == 1 ) then
		EkInspect_Frame_UISpecialFrame(1);
		EkInspect_Frame_UIPanelSet(0);
		EkInspect_Frame_MoveButton:Show();
	else
		EkInspect_Frame_MoveButton:Hide();
		EkInspect_Frame_UISpecialFrame(0);
		EkInspect_Frame_UIPanelSet(1);
	end
	EkInspect_Frame_UIPanelShow(EkInspect_Frame_);
end

function EkInspect_Frame_UIPanelSet(set)
	-- --------------
	-- Set or clear standard UI Panel settings for the window.
	-- This allows the frame to act like a standard UI panel (shift right when you open another)
	-- when ShowUIPanel() and HideUIPanel() are used.
	-- --------------
	local frame = EkInspect_Frame_;

	if (set == 1) then
		-- UIPanelWindows["EkInspect_Frame_"] = { area = "left", pushable = 1, whileDead = 1 };

		frame:SetAttribute("UIPanelLayout-defined", true);
		frame:SetAttribute("UIPanelLayout-area", "left");
		frame:SetAttribute("UIPanelLayout-pushable", 1);
		frame:SetAttribute("UIPanelLayout-whileDead", 1);
		frame:SetAttribute("UIPanelLayout-enabled", true);
	else
		-- UIPanelWindows["EkInspect_Frame_"] = nil;

		frame:SetAttribute("UIPanelLayout-enabled", false);
	end
end

function EkInspect_Frame_UIPanelShow(frame)
	-- ----------
	-- Show the frame.
	-- ----------
	ShowUIPanel(frame);
--	frame:Show();
end

function EkInspect_Frame_UIPanelHide(frame)
	-- ----------
	-- Hide the frame.
	-- ----------
	HideUIPanel(frame);
--	frame:Hide();
end

function EkInspect_Frame_UISpecialFrame(set)
	-- --------------
	-- Set or clear UI Special Frame settings for the window.
	-- (this allows you to press ESC to close the frame).
	-- --------------
	if (set == 1) then
		tinsert(UISpecialFrames, "EkInspect_Frame_");
	else
		for key, val in pairs(UISpecialFrames) do
			if ( key == "EkInspect_Frame_" ) then
				val = nil;
			end
		end
	end
end

function EkInspect_Frame_ShowSubFrame(frameName)
	-- ----------
	-- Show/hide the sub frames.
	-- ----------
	EkInspect_Frame_ShowSubFrameTable(frameName, EkInspect_Frame_SUBFRAMES);
end

function EkInspect_Frame_ShowSubFrameTable(frameName, frameNameTable)
	-- ----------
	-- Show/hide the sub frame names listed in a table.
	-- ----------
	local frame;
	for index, value in pairs(frameNameTable) do
		if ( type(value) == "table" ) then
			EkInspect_Frame_ShowSubFrameTable(frameName, value);
		else
			frame = _G[value];
			if ( frame ) then
				if ( value == frameName ) then
					frame:Show();
				else
					frame:Hide();
				end
			end
		end
	end
end

function EkInspect_Frame_HideSubFrames()
	-- ----------
	-- Hide the sub frames.
	-- ----------
	EkInspect_Frame_HideSubFrameTable(EkInspect_Frame_SUBFRAMES);
end

function EkInspect_Frame_HideSubFrameTable(frameNameTable)
	-- ----------
	-- Hide the sub frame names listed in a table.
	-- ----------
	local frame;
	for index, value in pairs(frameNameTable) do
		if ( type(value) == "table" ) then
			EkInspect_Frame_HideSubFrameTable(value);
		else
			frame = _G[value];
			if ( frame ) then
				frame:Hide();
			end
		end
	end
end

function EkInspect_Frame_OnLoad(self)
	-- ----------
	-- Frame has been loaded.
	-- ----------
	local frame = EkInspect_Frame_;

	PanelTemplates_SetNumTabs(self, #EkInspect_Frame_SUBFRAMES);

	-- Initial tab to be selected
	frame.selectedTab = EkInspect_Frame_TabMain;

	PanelTemplates_UpdateTabs(self);
end

function EkInspect_Frame_OnShow()
	-- ----------
	-- Show the frame.
	-- ----------
	local fname = "EkInspect_Frame_";
	local frame = _G[fname];
	local tab = frame.selectedTab;

	if (tab == EkInspect_Frame_TabOptions) then
		fname = EkInspect_Frame_SUBFRAMES[tab][EkInspect_OptionsFrame_Page];
	else
		fname = EkInspect_Frame_SUBFRAMES[tab];
	end

	EkInspect_Frame_ShowSubFrame(fname);
	PlaySound("igMainMenuOpen");
end

function EkInspect_Frame_GetTab()
	-- ----------
	-- Get current tab number.
	-- ----------
	local frame = EkInspect_Frame_;
	local tab = frame.selectedTab;
	return tab;
end

function EkInspect_Frame_Refresh()
	-- ----------
	-- Refresh the frame.
	-- (also called from EkInspect.lua).
	-- ----------
	local frame = EkInspect_Frame_;
	local tab = frame.selectedTab;

	if ( tab == EkInspect_Frame_TabOptions ) then
		EkInspect_OptionsFrame_OnShow();

	elseif ( tab == EkInspect_Frame_TabMain ) then
		EkInspect_MainFrame_OnShow();

	elseif ( tab == EkInspect_Frame_TabSummary ) then
		EkInspect_SummaryFrame_OnShow();

	elseif ( tab == EkInspect_Frame_TabDetails ) then
		EkInspect_DetailsFrame_OnShow();

	end
end

function EkInspect_Frame_OnHide(self)
	-- ----------
	-- Hide the frame.
	-- ----------
	PlaySound("igMainMenuClose");
	EkInspect_Frame_HideSubFrames();
	self:StopMovingOrSizing();
end

function EkInspect_Frame_Tab_OnClick(self)
	-- ----------
	-- User clicked a tab on the frame.
	-- ----------
	PanelTemplates_Tab_OnClick(self, EkInspect_Frame_);
	EkInspect_Frame_OnShow();
end

function EkInspect_Frame_GotoTab(tab)
	-- ----------
	-- Display one of the UI tab pages.
	-- ----------
	local frame = EkInspect_Frame_;

	PanelTemplates_SetTab(frame, tab);
	EkInspect_Frame_OnShow();
end

-- *!*
-- -------------------------------------------------------------------------
-- EkInspect_OptionsFrame (the "options" tab)
-- -------------------------------------------------------------------------

EkInspect_OptionsFrame_Page = 1;

function EkInspect_OptionsFrame_OnLoad()
	-- ----------
	-- Options frame is being loaded.
	-- ----------
end

function EkInspect_OptionsFrame_OnShow()
	-- ----------
	-- Show the frame.
	-- ----------

	if (EkInspect_OptionsFrame_Page == 1) then
		EkInspect_OptionsFrame_TitleText:SetText("EkInspect " .. EkInspect_Version);

		EkInspect_OptionsFrame_MovableCB_Update();
		EkInspect_OptionsFrame_ShowTooltipsCB:SetChecked(EkInspect_ShowTooltips);
		EkInspect_OptionsFrame_ShowAtCursorCB:SetChecked(EkInspect_ShowAtCursor);
		EkInspect_OptionsFrame_ShowAtCursorCB_Update();

	end

	EkInspect_OptionsFrame_ClearFocus();
end

function EkInspect_OptionsFrame_ClearFocus()
	-- ----------
	-- Clear focus from edit boxes
	-- ----------
	-- StartEB:ClearFocus();
end

-- ----------
-- "Movable window"
-- ----------

function EkInspect_OptionsFrame_MovableCB_OnClick()
	-- ----------
	-- Toggle the "movable window" option.
	-- ----------
	local CB = EkInspect_OptionsFrame_MovableCB;

	if ( CB:GetChecked() ) then
		EkInspect_MovableWindow = 1;
	else
		EkInspect_MovableWindow = 0;
	end

	EkInspect_Frame_UnlockWindow(EkInspect_MovableWindow);

	EkInspect_OptionsFrame_ClearFocus();
end

function EkInspect_OptionsFrame_MovableCB_Update()
	-- ----------
	-- Update the "movable window" checkbox.
	-- ----------
	local CB = EkInspect_OptionsFrame_MovableCB;

	CB:SetChecked(EkInspect_MovableWindow);
end

-- ----------
-- "Show detailed tooltips"
-- ----------

function EkInspect_OptionsFrame_ShowTooltipsCB_OnClick()
	-- ----------
	-- Toggle the "show detailed tooltips" option.
	-- ----------
	local CB = EkInspect_OptionsFrame_ShowTooltipsCB;

	if ( CB:GetChecked() ) then
		EkInspect_ShowTooltips = 1;
	else
		EkInspect_ShowTooltips = 0;
	end

	EkInspect_OptionsFrame_ShowAtCursorCB_Update();
	EkInspect_OptionsFrame_ClearFocus();
end

function EkInspect_OptionsFrame_ShowTooltipsCB_Update()
	-- ----------
	-- Update the "show detailed tooltips" checkbox.
	-- ----------
	local CB = EkInspect_OptionsFrame_ShowTooltipsCB;

	CB:SetChecked(EkInspect_ShowTooltips);
end

-- ----------
-- "Show tooltips at cursor"
-- ----------

function EkInspect_OptionsFrame_ShowAtCursorCB_OnClick()
	-- ----------
	-- Toggle the "show tooltips at cursor" option.
	-- ----------
	local CB = EkInspect_OptionsFrame_ShowAtCursorCB;

	if ( CB:GetChecked() ) then
		EkInspect_ShowAtCursor = 1;
	else
		EkInspect_ShowAtCursor = 0;
	end

	EkInspect_OptionsFrame_ClearFocus();
end

function EkInspect_OptionsFrame_ShowAtCursorCB_Update()
	-- ----------
	-- Update the "show tooltips at cursor" checkbox.
	-- ----------
	if (EkInspect_ShowTooltips == 1) then
		EkInspect_OptionsFrame_ShowAtCursorCB:Enable();
	else
		EkInspect_OptionsFrame_ShowAtCursorCB:Disable();
	end
end

-- *!*
-- -------------------------------------------------------------------------
-- EkInspect_MainFrame
-- -------------------------------------------------------------------------

function EkInspect_MainFrame_OnLoad(self)
	-- ----------
	-- Main frame is loading.
	-- ----------
	local col, row, xoff, yoff, cb, rb;

	-- Sort by slot name
	sort(EkInspect_TEXT_SlotNames, function(a, b) return strlower(a.name) < strlower(b.name) end);

	col = 0;
	row = 0;
	xoff = 30;
	yoff = 95;
	for i = 1, #EkInspect_TEXT_Methods do
		local rb = CreateFrame("CheckButton", "EkInspect_MainFrame_WhoToInspectRB" .. i, self, "EkInspect_RadioButtonTemplate");
		rb:SetID(i);
		_G[rb:GetName() .. "Name"]:SetText(EkInspect_TEXT_Methods[i]);
		rb:SetPoint("TOPLEFT", self, "TOPLEFT", xoff + col * 150, -(yoff + row * 20));
		col = col + 1;
		if (col >= 2) then
			col = 0;
			row = row + 1;
		end
		rb:Show();
	end

	col = 0;
	row = 0;
	xoff = 30;
	yoff = 170;
	for i = 0, #EkInspect_TEXT_SlotNames do
		local cb = CreateFrame("CheckButton", "EkInspect_MainFrame_WhatToInspectCB" .. i, self, "EkInspect_SlotCheckBoxTemplate");
		if (i == 0) then
			cb:SetID(0);
			_G[cb:GetName() .. "Name"]:SetText(EkInspect_TEXT_AllSlots);
		else
			local slotNames = EkInspect_TEXT_SlotNames[i];
			local id = slotNames.id;
			cb:SetID(id);
			if (id == 17) then  -- offhand slot
				_G[cb:GetName() .. "Name"]:SetText(slotNames.name .. " *");
--			elseif (id == 18) then -- ranged slot
--				_G[cb:GetName() .. "Name"]:SetText(slotNames.name .. " *");
			else
				_G[cb:GetName() .. "Name"]:SetText(slotNames.name);
			end
		end
		cb:SetPoint("TOPLEFT", self, "TOPLEFT", xoff + col * 150, -(yoff + row * 20));
		col = col + 1;
		if (col >= 2) then
			col = 0;
			row = row + 1;
		end
		cb:Show();
	end
end

function EkInspect_MainFrame_OnShow()
	-- ----------
	-- Show main frame.
	-- ----------
	EkInspect_MainFrame_TitleText:SetText(EkInspect_TEXT_Main_Title);
	EkInspect_MainFrame_SubTitle:SetText(string.format(EkInspect_TEXT_Main_Command_List, EkInspect_Slash1 .. " " .. EkInspect_TEXT_Command_Help));
	EkInspect_MainFrame_Refresh();
end

function EkInspect_MainFrame_Refresh()
	-- ----------
	-- Refresh frame contents.
	-- ----------
	EkInspect_MainFrame_WhoToInspect_Update();
	EkInspect_MainFrame_WhatToInspectText_Update();
	EkInspect_MainFrame_WhatToInspectCB_Update();
	EkInspect_MainFrame_AdjustItemLevelsCB_Update();

	local slots = EkInspect_UI_Slots[EkInspect_UI_Method];
	if (slots["all"]) then
		EkInspect_MainFrame_SelectAllButton:Disable();
		EkInspect_MainFrame_UnselectAllButton:Disable();
	else
		EkInspect_MainFrame_SelectAllButton:Enable();
		EkInspect_MainFrame_UnselectAllButton:Enable();
	end
end

function EkInspect_MainFrame_OnHide()
	-- ----------
	-- Frame is being hidden.
	-- ----------
end

function EkInspect_MainFrame_ClearFocus()
	-- ----------
	-- Clear focus from edit boxes
	-- ----------
	-- abc:ClearFocus();
end

function EkInspect_MainFrame_WhoToInspectRB_OnLoad(self)
	_G[self:GetName() .. "Name"]:SetText(EkInspect_TEXT_Methods[self:GetID()]);
end

function EkInspect_MainFrame_WhoToInspect_Update()
	-- ----------
	-- Update the "Who to inspect" radio buttons.
	-- ----------
	for i = 1, #EkInspect_TEXT_Methods do
		local rb = _G["EkInspect_MainFrame_WhoToInspectRB" .. i];
		if (EkInspect_UI_Method == i) then
			rb:SetChecked(true);
		else
			rb:SetChecked(false);
		end
		rb:Show();
	end
end

function EkInspect_MainFrame_WhoToInspectRB_OnClick(self)
	-- ----------
	-- User clicked a "Who to inspect" radio button.
	-- ----------
	EkInspect_UI_Method = self:GetID();
	EkInspect_MainFrame_Refresh();
end

function EkInspect_MainFrame_WhatToInspectText_Update()
	-- ----------
	-- Update the "What to inspect" text.
	-- ----------
	EkInspect_MainFrame_SlotsText:SetText(format(EkInspect_TEXT_MainFrame_Slots, EkInspect_TEXT_Methods[EkInspect_UI_Method]));
end

function EkInspect_MainFrame_WhatToInspectCB_Update()
	-- ----------
	-- Update the "What to inspect" check boxes.
	-- ----------
	local slots = EkInspect_UI_Slots[EkInspect_UI_Method];
	local cb;

	cb = EkInspect_MainFrame_WhatToInspectCB0;
	if (slots["all"]) then
		cb:SetChecked(true);
	else
		cb:SetChecked(false);
	end
	cb:Show();

	for i = 1, #EkInspect_TEXT_SlotNames do
		local slotNames = EkInspect_TEXT_SlotNames[i];
		local cb = _G["EkInspect_MainFrame_WhatToInspectCB" .. i];
		if (slots[slotNames.id]) then
			cb:SetChecked(true);
		else
			cb:SetChecked(false);
		end
		if (slots["all"]) then
			cb:Disable();
			-- cb:Hide();
			cb:Show();
		else
			cb:Enable();
			cb:Show();
		end
	end
end

function EkInspect_MainFrame_WhatToInspectCB_OnClick(self)
	-- ----------
	-- User clicked a "What to inspect" check box.
	-- ----------
	local slots = EkInspect_UI_Slots[EkInspect_UI_Method];
	local id = self:GetID();
	if (id == 0) then
		slots["all"] = self:GetChecked();
	else
		slots[id] = self:GetChecked();
	end
	EkInspect_MainFrame_Refresh();

	if (EkInspect_UI_Method == EkInspect_WhoWasInspected) then
		EkInspect_Refilter = true;
	end
end

function EkInspect_MainFrame_WhatToInspectCB_OnEnter(self)
	-- ----------
	-- Mouse is over a "What to inspect" check box.
	-- ----------
	local id = self:GetID();
	if (id == 0) then
		EkInspect_SetTooltip_Corner(self, EkInspect_TEXT_AllSlots, EkInspect_TEXT_AllSlots_Tooltip2);
		return;
	end
	for i = 1, #EkInspect_TEXT_SlotNames do
		local slotName = EkInspect_TEXT_SlotNames[i];
		if (slotName.id == id) then
			if (slotName.tip) then
				EkInspect_SetTooltip_Corner(self, slotName.name, slotName.tip);
				break;
			end
		end
	end
end


function EkInspect_MainFrame_AdjustItemLevelsCB_OnClick(self)
	-- ----------
	-- Adjust item levels checkbox.
	-- ----------
	if (self:GetChecked()) then
		EkInspect_UI_Adjust[EkInspect_UI_Method] = 1;
	else
		EkInspect_UI_Adjust[EkInspect_UI_Method] = 0;
	end

	EkInspect_MainFrame_Refresh();

	if (EkInspect_UI_Method == EkInspect_WhoWasInspected) then
		EkInspect_Refilter = true;
	end
end

function EkInspect_MainFrame_AdjustItemLevelsCB_Update()
	local cb = EkInspect_MainFrame_AdjustItemLevelsCB;
	if (EkInspect_UI_Adjust[EkInspect_UI_Method] == 1) then
		cb:SetChecked(true);
	else
		cb:SetChecked(false);
	end
	cb:Show();
end

-- ----------
-- Select all button
-- ----------
function EkInspect_MainFrame_SelectAllButton_OnClick(self)
	-- ----------
	-- Select all slot checkboxes
	-- ----------
	EkInspect_MainFrame_ClearFocus();
	local slots = EkInspect_UI_Slots[EkInspect_UI_Method];
	for i = 1, #EkInspect_TEXT_SlotNames do
		local slotName = EkInspect_TEXT_SlotNames[i];
		slots[slotName.id] = 1;
	end
	EkInspect_MainFrame_Refresh();

	if (EkInspect_UI_Method == EkInspect_WhoWasInspected) then
		EkInspect_Refilter = true;
	end
end

function EkInspect_MainFrame_SelectAllButton_OnEnter(self)
	EkInspect_SetTooltip_Corner(self, EkInspect_TEXT_Main_SelectAll_Tooltip1, EkInspect_TEXT_Main_SelectAll_Tooltip2);
end

-- ----------
-- Unselect all button
-- ----------
function EkInspect_MainFrame_UnselectAllButton_OnClick(self)
	-- ----------
	-- Unselect all slot checkboxes
	-- ----------
	EkInspect_MainFrame_ClearFocus();
	local slots = EkInspect_UI_Slots[EkInspect_UI_Method];
	for i = 1, #EkInspect_TEXT_SlotNames do
		local slotName = EkInspect_TEXT_SlotNames[i];
		slots[slotName.id] = nil;
	end
	EkInspect_MainFrame_Refresh();

	if (EkInspect_UI_Method == EkInspect_WhoWasInspected) then
		EkInspect_Refilter = true;
	end
end

function EkInspect_MainFrame_UnselectAllButton_OnEnter(self)
	EkInspect_SetTooltip_Corner(self, EkInspect_TEXT_Main_UnselectAll_Tooltip1, EkInspect_TEXT_Main_UnselectAll_Tooltip2);
end


-- ----------
-- Inspect button
-- ----------

function EkInspect_MainFrame_InspectButton_OnClick()
	-- ----------
	-- User clicked on the inspect button.
	-- ----------
	EkInspect_MainFrame_ClearFocus();
	EkInspect_Inspect_Perform(EkInspect_UI_Method, nil);
	EkInspect_Frame_GotoTab(EkInspect_Frame_TabSummary);
end

function EkInspect_MainFrame_InspectButton_OnEnter(self)
	-- ----------
	-- Mouse is over the inspect button.
	-- ----------
	EkInspect_SetTooltip_Corner(self, EkInspect_TEXT_Main_Inspect_Tooltip1, EkInspect_TEXT_Main_Inspect_Tooltip2);
end


-- *!*
-- -------------------------------------------------------------------------
-- EkInspect_SummaryFrame
-- -------------------------------------------------------------------------

EkInspect_SummaryFrame_MenuNum = 1;
EkInspect_SummaryFrame_Order = "level";
EkInspect_SummaryFrame_LastSort = EkInspect_SummaryFrame_Order;  -- Last sort type
EkInspect_SummaryFrame_Ascend = nil;  -- nil=Descending, 1=Ascending
EkInspect_SummaryFrame_NumToDisplay = 17;
EkInspect_SummaryFrame_Height = 16;
EkInspect_SummaryFrame_Selected = {};
EkInspect_SummaryFrame_SelectedTemp = {};

function EkInspect_ScrollBar_OnMouseWheel(scrollbar, direction)
	-- ----------
	-- User scrolled mouse wheel.
	-- ----------
	local sb = scrollbar;
	if (direction == 1) then
		-- Scrolled forward
		sb:SetValue(sb:GetValue() - (sb.ekheight / 2));
		PlaySound("UChatScrollButton");
	else
		-- Scrolled backward
		sb:SetValue(sb:GetValue() + (sb.ekheight / 2));
		PlaySound("UChatScrollButton");
	end
end

function EkInspect_SummaryFrame_OnLoad()
	-- ----------
	-- Load frame.
	-- ----------
end

function EkInspect_SummaryFrame_OnShow()
	-- ----------
	-- Frame is being shown.
	-- ----------
	local slots = EkInspect_UI_Slots[EkInspect_WhoWasInspected];

	EkInspect_SummaryFrame_TitleText:SetText(EkInspect_TEXT_Summary_Title);

--	EkInspect_Frame_MenuButton_HideAll();

	if (EkInspect_Refilter) then
		EkInspect_Refilter = false;
		EkInspect_Inspect_ApplyFilter();
	end

	EkInspect_SummaryFrame_ListUpdate();
	EkInspect_SummaryFrame_ShowMenu();
end

function EkInspect_SummaryFrame_OnHide()
	-- ----------
	-- Frame is being hidden.
	-- ----------
end

function EkInspect_SummaryFrame_Refresh()
	-- ----------
	-- Refresh the summary frame contents.
	-- ----------
	if (EkInspect_SummaryFrame_:IsShown()) then
		EkInspect_SummaryFrame_ListUpdate();
		EkInspect_SummaryFrame_ShowMenu();
	end
end

function EkInspect_SummaryFrame_ListInit()
	-- ----------
	-- Initialize the array to be displayed.
	-- ----------
	EkInspect_SummaryFrame_Selected = {};
	EkInspect_SummaryFrame_Sort(EkInspect_SummaryFrame_Order);
end

function EkInspect_SummaryFrame_SelToTemp()
	-- ----------
	-- Convert current selection array (containing numeric subscripts)
	-- into a temporary selection array (containing index strings).
	-- ----------
	local list = EkInspect_SummaryList;
	local sel = EkInspect_SummaryFrame_Selected;
	local sel2;
	local sub, len;

	sel2 = {};  -- temp selection array.
	len = #sel;
	for i=1, len do
		sub = sel[i];
		table.insert(sel2, list[sub]);
	end

	EkInspect_SummaryFrame_SelectedTemp = sel2;
end

function EkInspect_SummaryFrame_TempToSel()
	-- ----------
	-- Convert temporary selection array (containing index strings)
	-- into a new current selection array (containing numeric subscripts).
	-- ----------
	local list = EkInspect_SummaryList;
	local sel = EkInspect_SummaryFrame_Selected;
	local sel2 = EkInspect_SummaryFrame_SelectedTemp;
	local sub, len, len2, count, index;

	len = #list;
	len2 = #sel2;
	sel = {};  -- start a new selection set
	if (len2 > 0) then
		count = 0;
		for i=1, len do
			index = list[i];
			-- Look up the index string in the temp selection array.
			for j=1, len2 do
				if (index == sel2[j]) then
					table.insert(sel, i);
					count = count + 1;
					break;
				end
			end
			-- Check if we've converted them all yet.
			if (count == len2) then
				break;
			end
		end
	end
	EkInspect_SummaryFrame_Selected = sel;
end

function EkInspect_SummaryFrame_Sort(sortType)
	-- --------
	-- Sort summary list.
	--
	-- Param: sortType == "name".
	-- --------
	local list = EkInspect_SummaryList;
	local ch, chNum;

	EkInspect_SummaryFrame_SelToTemp();

	for i = 1, 4 do
		ch = _G["EkInspect_SummaryFrame_ColumnHeader" .. i];
		EkInspect_SetButtonTextColor(ch, 1, 1, 1, 1);
		ch:Hide();
		ch:Show();
	end
		
	if (sortType == "name") then
		sort(list, EkInspect_SummaryFrame_SortByName);
		chNum = 1;
	elseif (sortType == "level") then
		sort(list, EkInspect_SummaryFrame_SortByLevel);
		chNum = 2;
	elseif (sortType == "average") then
		sort(list, EkInspect_SummaryFrame_SortByAverage);
		chNum = 3;
	elseif (sortType == "class") then
		sort(list, EkInspect_SummaryFrame_SortByClass);
		chNum = 4;
	else
		sort(list, EkInspect_SummaryFrame_SortByName);
		chNum = 1;
	end

	ch = _G["EkInspect_SummaryFrame_ColumnHeader" .. chNum];
	EkInspect_SetButtonTextColor(ch, 1, 0.82, 0, 1);
	ch:Hide();
	ch:Show();

	EkInspect_SummaryFrame_TempToSel();

	EkInspect_SummaryFrame_Order = sortType;
	EkInspect_SummaryFrame_LastSort = sortType;
end

function EkInspect_SummaryFrame_SortByName(val1, val2)
	-- ----------
	-- Sort the list by name.
	-- ----------
	local order, low1, low2;

	low1 = strlower_utf8(val1);
	low2 = strlower_utf8(val2);

	if ( low1 < low2 ) then
		order = -1;
	elseif ( low1 > low2 ) then
		order = 1;
	else
		order = 0;
	end

	if (EkInspect_SummaryFrame_Ascend) then
		if (order < 0) then
			return true;
		else
			return false;
		end
	else
		if (order > 0) then
			return true;
		else
			return false;
		end
	end
end

function EkInspect_SummaryFrame_SortByClass(val1, val2)
	-- ----------
	-- Sort the list by class.
	-- ----------
	local order, low1, low2;

	local playerTable1 = EkInspect_InspectList[val1];
	local playerTable2 = EkInspect_InspectList[val2];

	low1 = strlower(playerTable1["class"]);
	low2 = strlower(playerTable2["class"]);

	if ( low1 < low2 ) then
		order = -1;
	elseif ( low1 > low2 ) then
		order = 1;
	else
		low1 = playerTable1["total"];
		low2 = playerTable2["total"];

		if ( low1 < low2 ) then
			order = -1;
		elseif ( low1 > low2 ) then
			order = 1;
		else
			low1 = strlower_utf8(val1);
			low2 = strlower_utf8(val2);

			if ( low1 < low2 ) then
				order = -1;
			elseif ( low1 > low2 ) then
				order = 1;
			else
				order = 0;
			end
		end
	end

	if (EkInspect_SummaryFrame_Ascend) then
		if (order < 0) then
			return true;
		else
			return false;
		end
	else
		if (order > 0) then
			return true;
		else
			return false;
		end
	end
end

function EkInspect_SummaryFrame_SortByLevel(val1, val2)
	-- ----------
	-- Sort the list by total.
	-- ----------
	local order, low1, low2;

	local playerTable1 = EkInspect_InspectList[val1];
	local playerTable2 = EkInspect_InspectList[val2];

	low1 = playerTable1["total"];
	low2 = playerTable2["total"];

	if ( low1 < low2 ) then
		order = -1;
	elseif ( low1 > low2 ) then
		order = 1;
	else
		low1 = strlower_utf8(val1);
		low2 = strlower_utf8(val2);

		if ( low1 < low2 ) then
			order = -1;
		elseif ( low1 > low2 ) then
			order = 1;
		else
			order = 0;
		end
	end

	if (EkInspect_SummaryFrame_Ascend) then
		if (order < 0) then
			return true;
		else
			return false;
		end
	else
		if (order > 0) then
			return true;
		else
			return false;
		end
	end
end

function EkInspect_SummaryFrame_SortByAverage(val1, val2)
	-- ----------
	-- Sort the list by average.
	-- ----------
	local order, low1, low2;

	local playerTable1 = EkInspect_InspectList[val1];
	local playerTable2 = EkInspect_InspectList[val2];

	low1 = playerTable1["average"];
	low2 = playerTable2["average"];

	if ( low1 < low2 ) then
		order = -1;
	elseif ( low1 > low2 ) then
		order = 1;
	else
		low1 = strlower_utf8(val1);
		low2 = strlower_utf8(val2);

		if ( low1 < low2 ) then
			order = -1;
		elseif ( low1 > low2 ) then
			order = 1;
		else
			order = 0;
		end
	end

	if (EkInspect_SummaryFrame_Ascend) then
		if (order < 0) then
			return true;
		else
			return false;
		end
	else
		if (order > 0) then
			return true;
		else
			return false;
		end
	end
end

function EkInspect_SummaryFrame_ScrollBarUpdate()
	-- ----------
	-- Update the scroll bar.
	-- ----------
	local scrollBar = EkInspect_SummaryFrame_ScrollBar;
	local list = EkInspect_SummaryList;
	local numItems = #list;
	local showScrollBar = nil;
	local itemOffset;

	if ( numItems > EkInspect_SummaryFrame_NumToDisplay ) then
		showScrollBar = 1;
		scrollBar:SetMinMaxValues(1, numItems - EkInspect_SummaryFrame_NumToDisplay + 1)
		scrollBar:Show();
		itemOffset = scrollBar:GetValue() - 1;
	else
		scrollBar:SetMinMaxValues(1, 1);
		scrollBar:Hide();
		itemOffset = 0;
	end

	scrollBar.ekoffset = itemOffset;
	return itemOffset, showScrollBar;
end

function EkInspect_SummaryFrame_ListUpdate()
	-- ----------
	-- Update the displayed items.
	-- ----------
	local scrollBar = EkInspect_SummaryFrame_ScrollBar;
	local fs;
	local button;
	local list = EkInspect_SummaryList;
	local rec;
	local playerName;
	local color;

	local numItems = #list;

	local itemIndex;
	local len, pos, msg;
	local sel = EkInspect_SummaryFrame_Selected;

	local itemOffset;
	local showScrollBar = nil;

	itemOffset, showScrollBar = EkInspect_SummaryFrame_ScrollBarUpdate();

	msg = string.format(EkInspect_TEXT_SummaryList_Total_Inspected, numItems, EkInspect_InspectedMax);
	msg = msg .. "   " .. string.format(EkInspect_TEXT_SummaryList_Total_Average, EkInspect_Summary_Average);
	EkInspect_SummaryFrame_Totals:SetText(msg);

	for i=1, EkInspect_SummaryFrame_NumToDisplay, 1 do
		itemIndex = itemOffset + i;
		button = _G["EkInspect_SummaryFrame_Button"..i];
		button.itemIndex = itemIndex;

		if ( showScrollBar ) then
			button:SetWidth(298);
		else
			button:SetWidth(330);
		end

		if (itemIndex > 0 and itemIndex <= numItems) then
			playerName = list[itemIndex];

			rec = EkInspect_InspectList[playerName];
			color = RAID_CLASS_COLORS[rec.class2];
			if (not color) then
				color = { r = 1.0, g = 1.0, b = 1.0 };
			end

			fs = _G["EkInspect_SummaryFrame_Button"..i.."Name"];
			fs:SetTextColor(color.r, color.g, color.b, 1);
			fs:SetText(playerName);
			fs:Show();

			fs = _G["EkInspect_SummaryFrame_Button"..i.."Total"];
			fs:SetTextColor(color.r, color.g, color.b, 1);
			fs:SetText(rec["total"]);
			fs:Show();

			fs = _G["EkInspect_SummaryFrame_Button"..i.."Average"];
			fs:SetTextColor(color.r, color.g, color.b, 1);
			fs:SetText(format("%.2f", rec["average"]));
			fs:Show();

			fs = _G["EkInspect_SummaryFrame_Button"..i.."Class"];
			fs:SetTextColor(color.r, color.g, color.b, 1);
			fs:SetText(rec["class"]);
			fs:Show();

			-- If need scrollbar resize columns
			if ( showScrollBar ) then
				fs:SetWidth(70);
			else
				fs:SetWidth(90);
			end

			-- Highlight the correct name
			len = #sel;
			pos = 0;
			for i=1, len, 1 do
				if ( sel[i] == itemIndex ) then
					pos = i;
					break;
				end
			end
			if ( pos > 0 ) then
				button:LockHighlight();
			else
				button:UnlockHighlight();
			end

			button:Show();
		else
			button:Hide();
		end
	end

	scrollBar:SetValue(scrollBar:GetValue());
end

function EkInspect_SummaryFrame_Column_SetWidth(frame, width)
	-- ----------
	-- Set width of a column.
	-- ----------
	frame:SetWidth(width);
	_G[frame:GetName().."Middle"]:SetWidth(width - 9);
end

function EkInspect_SummaryFrame_Column_Click(sortType)
	-- ----------
	-- User clicked on a column heading.
	-- ----------
	if ( sortType ) then
		if (sortType == EkInspect_SummaryFrame_LastSort) then
			if (EkInspect_SummaryFrame_Ascend) then
				EkInspect_SummaryFrame_Ascend = nil;
			else
				EkInspect_SummaryFrame_Ascend = 1;
			end
		end

		EkInspect_SummaryFrame_Sort(sortType);
		EkInspect_SummaryFrame_ListUpdate();
	end
	PlaySound("igMainMenuOptionCheckBoxOn");
end

function EkInspect_SummaryFrame_Button_OnClick(self, button)
	-- ----------
	-- User clicked on a player.
	-- ----------
	local sel = EkInspect_SummaryFrame_Selected; -- Table of selected subscripts
	local sub = self.itemIndex; -- Subscript stored in this button
	local sub1, sub2, sub3, pos, len;

	if ( button == "LeftButton" ) then

		if ( #sel == 0 ) then
			-- Select a single item. Cancels any existing selection.
			sel = { sub };

		elseif ( IsShiftKeyDown() ) then
			-- -----
			-- Select all items between the start and end points.
			-- -----
			sub1 = sel[1];
			sub2 = sel[#sel];
			if ( sub < sub1 ) then
				sub1 = sub;
			else
				sub2 = sub;
			end
			sel = {};
			len = sub2 - sub1 + 1;
			sub3 = sub1;
			for i=1, len, 1 do
				sel[i] = sub3;
				sub3 = sub3 + 1;
			end

		elseif ( IsControlKeyDown() ) then
			-- -----
			-- User control clicked an item.
			-- If already selected, unselect it,
			-- otherwise select it and add it to the selection list.
			-- -----
			len = #sel;
			pos = 0;
			for i=1, len, 1 do
				if ( sel[i] == sub ) then
					-- Already in the table, so remove it.
					pos = i;
					table.remove(sel, i);
					break;
				elseif ( sel[i] > sub ) then
					-- Not in table, so insert it at proper spot.
					pos = i;
					table.insert(sel, i, sub);
					break;
				end
			end
			-- If it still is not in the table, then add it to the end.
			if ( pos == 0 ) then
				table.insert(sel, sub);
			end
		else
			-- Select a single item. Cancels any existing selection.
			sel = { sub };
		end

		EkInspect_SummaryFrame_Selected = sel;
		-- EkInspect_SummaryFrame_MenuNum = 1;  -- SummaryFrame main menu
		EkInspect_SummaryFrame_Refresh();
	end
end

function EkInspect_SummaryFrame_Button_OnDoubleClick(self, button)
	-- ----------
	-- User double clicked on a player in the summary list.
	-- ----------
	local sel = EkInspect_SummaryFrame_Selected; -- Table of selected subscripts
	local sub = self.itemIndex; -- Subscript stored in this button

	if ( button == "LeftButton" ) then
		-- Update display.
		-- EkInspect_SummaryFrame_MenuNum = 1;
		-- EkInspect_SummaryFrame_Refresh();
	end
end

function EkInspect_SummaryFrame_Button_OnEnter(self)
	-- ----------
	-- User moved pointer over a player item.
	-- ----------
	local sel = EkInspect_SummaryFrame_Selected; -- Table of selected subscripts
	local sub = self.itemIndex; -- Subscript stored in this button

	local xp = "LEFT";
	local yp = "BOTTOM";
	local xo = 0;
	local xthis, ythis = self:GetCenter();
	local xui, yui = UIParent:GetCenter();
	if (xthis < xui) then
		xp = "RIGHT";
		if (#EkInspect_SummaryList <= EkInspect_SummaryFrame_NumToDisplay) then
			xo = 0;
		else
			xo = 32;
		end
	end
	if (ythis < yui) then
--		yp = "TOP";
		yp = "";
		if (xthis < xui) then
			xp = "RIGHT";
		else
			xp = "LEFT";
		end
	end

	local name = EkInspect_SummaryList[sub];
	local rec = EkInspect_InspectList[name];
	local text, white;

	GameTooltip:SetOwner(self, "ANCHOR_" .. yp .. xp, xo, 0);

	white = "|c00FFFFFF";

	local r, g, b, alpha = 1, 0.82, 0, 1;
	GameTooltip:ClearLines();
	GameTooltip:AddLine(white .. EkInspect_TEXT_Summary_Tooltip_Name .. "|r " .. name, r, g, b, alpha, 1);
	GameTooltip:AddLine(white .. EkInspect_TEXT_Summary_Tooltip_Class .. "|r " .. rec["class"], r, g, b, alpha, 1);
	GameTooltip:AddLine(white .. EkInspect_TEXT_Summary_Tooltip_Total .. "|r " .. rec["total"], r, g, b, alpha, 1);
	GameTooltip:AddLine(white .. EkInspect_TEXT_Summary_Tooltip_Average .. "|r " .. format("%.2f", rec["average"]), r, g, b, alpha, 1);
        GameTooltip:AddLine(" ", r, g, b, alpha, 1);
	GameTooltip:AddLine(white .. EkInspect_TEXT_Summary_Tooltip_Click1 .. "|r " .. EkInspect_TEXT_Summary_Tooltip_Click2, r, g, b, alpha, 1);
	GameTooltip:AddLine(white .. EkInspect_TEXT_Summary_Tooltip_Ctrl_Click1 .. "|r " .. EkInspect_TEXT_Summary_Tooltip_Ctrl_Click2, r, g, b, alpha, 1);
	GameTooltip:AddLine(white .. EkInspect_TEXT_Summary_Tooltip_Shift_Click1 .. "|r " .. EkInspect_TEXT_Summary_Tooltip_Shift_Click2, r, g, b, alpha, 1);
	-- GameTooltip:AddLine(white .. EkInspect_TEXT_Summary_Tooltip_Double_Click1 .. "|r " .. EkInspect_TEXT_Summary_Tooltip_Double_Click2, r, g, b, alpha, 1);

	GameTooltip:Show();
end

function EkInspect_SummaryFrame_ShowMenu()
	-- ----------
	-- Display the 'menu'.
	-- ----------
	local sel = EkInspect_SummaryFrame_Selected;

	EkInspect_SummaryFrame_SelectAllButton:Hide();
	EkInspect_SummaryFrame_UnselectAllButton:Hide();
	EkInspect_SummaryFrame_AnnounceButton:Hide();

	EkInspect_SummaryFrame_BackButton:Hide();
	EkInspect_SummaryFrame_PartyButton:Hide();
	EkInspect_SummaryFrame_RaidButton:Hide();
	EkInspect_SummaryFrame_OfficerButton:Hide();

	if (EkInspect_SummaryFrame_MenuNum == 2) then
		EkInspect_SummaryFrame_BackButton:Show();
		EkInspect_SummaryFrame_PartyButton:Show();
		EkInspect_SummaryFrame_RaidButton:Show();
		EkInspect_SummaryFrame_OfficerButton:Show();

		if (GetNumPartyMembers() > 0) then
			EkInspect_SummaryFrame_PartyButton:Enable();
		else
			EkInspect_SummaryFrame_PartyButton:Disable();
		end

		if (GetNumRaidMembers() > 0) then
			EkInspect_SummaryFrame_RaidButton:Enable();
		else
			EkInspect_SummaryFrame_RaidButton:Disable();
		end

		if (IsInGuild() and EkInspect_CanOfficerChat()) then
			EkInspect_SummaryFrame_OfficerButton:Enable();
		else
			EkInspect_SummaryFrame_OfficerButton:Disable();
		end
	else
		EkInspect_SummaryFrame_SelectAllButton:Show();
		EkInspect_SummaryFrame_UnselectAllButton:Show();
		EkInspect_SummaryFrame_AnnounceButton:Show();
	end
end

-- ----------
-- Select all button
-- ----------
function EkInspect_SummaryFrame_SelectAllButton_OnClick(self)
	-- ----------
	-- Select all slot checkboxes
	-- ----------
	local sel = {};
	for i=1, #EkInspect_SummaryList do
		sel[i] = i;
	end
	EkInspect_SummaryFrame_Selected = sel;
	EkInspect_SummaryFrame_Refresh();
end

function EkInspect_SummaryFrame_SelectAllButton_OnEnter(self)
	EkInspect_SetTooltip_Corner(self, EkInspect_TEXT_Summary_SelectAll_Tooltip1, EkInspect_TEXT_Summary_SelectAll_Tooltip2);
end

-- ----------
-- Unselect all button
-- ----------
function EkInspect_SummaryFrame_UnselectAllButton_OnClick(self)
	-- ----------
	-- Unselect all slot checkboxes
	-- ----------
	EkInspect_SummaryFrame_Selected = {};
	EkInspect_SummaryFrame_Refresh();
end

function EkInspect_SummaryFrame_UnselectAllButton_OnEnter(self)
	EkInspect_SetTooltip_Corner(self, EkInspect_TEXT_Summary_UnselectAll_Tooltip1, EkInspect_TEXT_Summary_UnselectAll_Tooltip2);
end


-- ----------
-- Announce button
-- ----------

function EkInspect_SummaryFrame_AnnounceButton_OnClick()
	-- ----------
	-- User clicked on the announce button.
	-- ----------
	EkInspect_SummaryFrame_MenuNum = 2;
	EkInspect_SummaryFrame_ShowMenu();
end

function EkInspect_SummaryFrame_AnnounceButton_OnEnter(self)
	-- ----------
	-- Mouse is over the announce button.
	-- ----------
	EkInspect_SetTooltip_Corner(self, EkInspect_TEXT_Summary_Announce_Tooltip1, EkInspect_TEXT_Summary_Announce_Tooltip2);
end


-- ----------
-- Officer button
-- ----------
function EkInspect_SummaryFrame_OfficerButton_OnClick(self)
	-- ----------
	-- Announce player information to the officer channel.
	-- ----------
	if (IsInGuild() and EkInspect_CanOfficerChat()) then
		EkInspect_SummaryFrame_AnnounceResults("OFFICER");
	end
	EkInspect_SummaryFrame_MenuNum = 1;
	EkInspect_SummaryFrame_ShowMenu();
end

-- ----------
-- Party button
-- ----------
function EkInspect_SummaryFrame_PartyButton_OnClick(self)
	-- ----------
	-- Announce player information to the party channel.
	-- ----------
	if (GetNumPartyMembers() > 0) then
		EkInspect_SummaryFrame_AnnounceResults("PARTY");
	end
	EkInspect_SummaryFrame_MenuNum = 1;
	EkInspect_SummaryFrame_ShowMenu();
end

-- ----------
-- Raid button
-- ----------
function EkInspect_SummaryFrame_RaidButton_OnClick(self)
	-- ----------
	-- Announce player information to the raid channel.
	-- ----------
	if (GetNumRaidMembers() > 0) then
		EkInspect_SummaryFrame_AnnounceResults("RAID");
	end
	EkInspect_SummaryFrame_MenuNum = 1;
	EkInspect_SummaryFrame_ShowMenu();
end

-- ----------
-- Back button
-- ----------
function EkInspect_SummaryFrame_BackButton_OnClick(self)
	-- ----------
	-- Move back to the main menu
	-- ----------
	EkInspect_SummaryFrame_MenuNum = 1;
	EkInspect_SummaryFrame_ShowMenu();
end

function EkInspect_SummaryFrame_AnnounceResults(channel)
	-- ----------------
	-- Announce selected players' results.
	-- ----------------
	local sel = EkInspect_SummaryFrame_Selected;

	if (#sel == 0) then
		for i, playerName in ipairs(EkInspect_SummaryList) do
			local playerTable = EkInspect_InspectList[playerName];
			EkInspect_SummaryFrame_AnnounceLine(format("%d. iLevel=%d, Avg=%.2f, %s.", i, playerTable.total, playerTable.average, playerName), channel);
		end
		EkInspect_SummaryFrame_AnnounceLine(format("Average iLevel=%.2f, %d players out of %d.", EkInspect_Summary_Average, #EkInspect_SummaryList, EkInspect_InspectedMax), channel);
	else
		local count = 0;
		local total = 0;
		for i=1, #sel do
			local sub = sel[i];
			local playerName = EkInspect_SummaryList[sub];
			local playerTable = EkInspect_InspectList[playerName];
			EkInspect_SummaryFrame_AnnounceLine(format("%d. iLevel=%d, Avg=%.2f, %s.", i, playerTable.total, playerTable.average, playerName), channel);
			if (playerTable.average ~= 0) then
				count = count + playerTable.total / playerTable.average;
			end
			total = total + playerTable.total;
		end
		local average = 0;
		if (count ~= 0) then
			average = total / count;
		end
		EkInspect_SummaryFrame_AnnounceLine(format("Average iLevel=%.2f.", average), channel);
	end
end

function EkInspect_SummaryFrame_AnnounceLine(msg, channel)
--	print(msg);
	SendChatMessage(msg, channel);
end


-- *!*
-- -------------------------------------------------------------------------
-- EkInspect_DetailsFrame
-- -------------------------------------------------------------------------

EkInspect_DetailsFrame_MenuNum = 1;
EkInspect_DetailsFrame_Order = "name";
EkInspect_DetailsFrame_LastSort = EkInspect_DetailsFrame_Order;  -- Last sort type
EkInspect_DetailsFrame_Ascend = 1;  -- nil=Descending, 1=Ascending
EkInspect_DetailsFrame_NumToDisplay = 20;
EkInspect_DetailsFrame_Height = 16;
EkInspect_DetailsFrame_Selected = {};
EkInspect_DetailsFrame_SelectedTemp = {};

function EkInspect_DetailsFrame_OnLoad()
	-- ----------
	-- Load frame.
	-- ----------
end

function EkInspect_DetailsFrame_OnShow()
	-- ----------
	-- Frame is being shown.
	-- ----------
	EkInspect_DetailsFrame_TitleText:SetText(EkInspect_TEXT_Details_Title);

--	EkInspect_Frame_MenuButton_HideAll();

	if (EkInspect_Refilter) then
		EkInspect_Refilter = false;
		EkInspect_Inspect_ApplyFilter();
	end

	EkInspect_DetailsFrame_ListUpdate();
--	EkInspect_DetailsFrame_ShowMenu();
end

function EkInspect_DetailsFrame_OnHide()
	-- ----------
	-- Frame is being hidden.
	-- ----------
end

function EkInspect_DetailsFrame_Refresh()
	-- ----------
	-- Refresh the frame contents.
	-- ----------
	if (EkInspect_DetailsFrame_:IsShown()) then
		EkInspect_DetailsFrame_ListUpdate();
--		EkInspect_DetailsFrame_ShowMenu();
	end
end

function EkInspect_DetailsFrame_ListInit()
	-- ----------
	-- Initialize the array to be displayed.
	-- ----------
	EkInspect_DetailsFrame_Selected = {};
	EkInspect_DetailsFrame_Sort(EkInspect_DetailsFrame_Order);
end

function  EkInspect_DetailsFrame_Sort(sortType)
	-- --------
	-- Sort the list.
	--
	-- Param: sortType == "name".
	-- --------
	local list = EkInspect_DetailsList;
	local ch, chNum;

--	EkInspect_DetailsFrame_SelToTemp();

	for i = 1, 3 do
		ch = _G["EkInspect_DetailsFrame_ColumnHeader" .. i];
		EkInspect_SetButtonTextColor(ch, 1, 1, 1, 1);
		ch:Hide();
		ch:Show();
	end
		
	if (sortType == "name") then
		sort(list, EkInspect_DetailsFrame_SortByName);
		chNum = 1;
	elseif (sortType == "level") then
		sort(list, EkInspect_DetailsFrame_SortByLevel);
		chNum = 2;
	elseif (sortType == "item") then
		sort(list, EkInspect_DetailsFrame_SortByItem);
		chNum = 3;
	else
		sort(list, EkInspect_DetailsFrame_SortByName);
		chNum = 1;
	end

	ch = _G["EkInspect_DetailsFrame_ColumnHeader" .. chNum];
	EkInspect_SetButtonTextColor(ch, 1, 0.82, 0, 1);
	ch:Hide();
	ch:Show();

--	EkInspect_DetailsFrame_TempToSel();

	EkInspect_DetailsFrame_Order = sortType;
	EkInspect_DetailsFrame_LastSort = sortType;
end

function EkInspect_DetailsFrame_SortByName(val1, val2)
	-- ----------
	-- Sort the list by name.
	-- ----------
	local order, low1, low2;

	local playerTable1 = EkInspect_InspectList[val1.name];
	local playerTable2 = EkInspect_InspectList[val2.name];

	local detailTable1 = playerTable1.details[val1.index];
	local detailTable2 = playerTable2.details[val2.index];

	low1 = strlower_utf8(val1.name);
	low2 = strlower_utf8(val2.name);

	if ( low1 < low2 ) then
		order = -1;
	elseif ( low1 > low2 ) then
		order = 1;
	else
		low1 = detailTable1["level"];
		low2 = detailTable2["level"];

		if ( low1 < low2 ) then
			order = -1;
		elseif ( low1 > low2 ) then
			order = 1;
		else
			low1 = strlower(detailTable1["item"]);
			low2 = strlower(detailTable2["item"]);

			if ( low1 < low2 ) then
				order = -1;
			elseif ( low1 > low2 ) then
				order = 1;
			else
				order = 0;
			end
		end
	end

	if (EkInspect_DetailsFrame_Ascend) then
		if (order < 0) then
			return true;
		else
			return false;
		end
	else
		if (order > 0) then
			return true;
		else
			return false;
		end
	end
end

function EkInspect_DetailsFrame_SortByLevel(val1, val2)
	-- ----------
	-- Sort the list by item level.
	-- ----------
	local order, low1, low2;

	local playerTable1 = EkInspect_InspectList[val1.name];
	local playerTable2 = EkInspect_InspectList[val2.name];

	local detailTable1 = playerTable1.details[val1.index];
	local detailTable2 = playerTable2.details[val2.index];

	low1 = detailTable1["level"];
	low2 = detailTable2["level"];

	if ( low1 < low2 ) then
		order = -1;
	elseif ( low1 > low2 ) then
		order = 1;
	else
		low1 = strlower_utf8(val1.name);
		low2 = strlower_utf8(val2.name);

		if ( low1 < low2 ) then
			order = -1;
		elseif ( low1 > low2 ) then
			order = 1;
		else
			low1 = strlower(detailTable1["item"]);
			low2 = strlower(detailTable2["item"]);

			if ( low1 < low2 ) then
				order = -1;
			elseif ( low1 > low2 ) then
				order = 1;
			else
				order = 0;
			end
		end
	end

	if (EkInspect_DetailsFrame_Ascend) then
		if (order < 0) then
			return true;
		else
			return false;
		end
	else
		if (order > 0) then
			return true;
		else
			return false;
		end
	end
end

function EkInspect_DetailsFrame_SortByItem(val1, val2)
	-- ----------
	-- Sort the list by item name.
	-- ----------
	local order, low1, low2;

	local playerTable1 = EkInspect_InspectList[val1.name];
	local playerTable2 = EkInspect_InspectList[val2.name];

	local detailTable1 = playerTable1.details[val1.index];
	local detailTable2 = playerTable2.details[val2.index];

	low1 = strlower(detailTable1["item"]);
	low2 = strlower(detailTable2["item"]);

	if ( low1 < low2 ) then
		order = -1;
	elseif ( low1 > low2 ) then
		order = 1;
	else
		low1 = strlower_utf8(val1.name);
		low2 = strlower_utf8(val2.name);

		if ( low1 < low2 ) then
			order = -1;
		elseif ( low1 > low2 ) then
			order = 1;
		else
			order = 0;
		end
	end

	if (EkInspect_DetailsFrame_Ascend) then
		if (order < 0) then
			return true;
		else
			return false;
		end
	else
		if (order > 0) then
			return true;
		else
			return false;
		end
	end
end

function EkInspect_DetailsFrame_ScrollBarUpdate()
	-- ----------
	-- Update the scroll bar.
	-- ----------
	local scrollBar = EkInspect_DetailsFrame_ScrollBar;
	local list = EkInspect_DetailsList;
	local numItems = #list;
	local showScrollBar = nil;
	local itemOffset;

	if ( numItems > EkInspect_DetailsFrame_NumToDisplay ) then
		showScrollBar = 1;
		scrollBar:SetMinMaxValues(1, numItems - EkInspect_DetailsFrame_NumToDisplay + 1)
		scrollBar:Show();
		itemOffset = scrollBar:GetValue() - 1;
	else
		scrollBar:SetMinMaxValues(1, 1);
		scrollBar:Hide();
		itemOffset = 0;
	end

	scrollBar.ekoffset = itemOffset;
	return itemOffset, showScrollBar;
end

function EkInspect_DetailsFrame_ListUpdate()
	-- ----------
	-- Update the displayed list.
	-- ----------
	local scrollBar = EkInspect_DetailsFrame_ScrollBar;
	local fs;
	local button;
	local list = EkInspect_DetailsList;
	local rec;
	local playerTable, detailTable;
	local r, g, b, hex;
	local color;

	local numItems = #list;

	local itemIndex;
	local len, pos, msg;
	local sel = EkInspect_DetailsFrame_Selected;

	local itemOffset;
	local showScrollBar = nil;

	itemOffset, showScrollBar = EkInspect_DetailsFrame_ScrollBarUpdate();

--	msg = string.format(EkInspect_TEXT_DetailsList_Total_Results, numItems);
--	if (#sel > 0) then
--		msg = msg .. "   " .. string.format(EkInspect_TEXT_DetailsList_Total_Selected, #sel);
--	end
--	EkInspect_DetailsFrame_Totals:SetText(msg);

	for i=1, EkInspect_DetailsFrame_NumToDisplay, 1 do
		itemIndex = itemOffset + i;
		button = _G["EkInspect_DetailsFrame_Button"..i];
		button.itemIndex = itemIndex;

		if ( showScrollBar ) then
			button:SetWidth(298);
		else
			button:SetWidth(330);
		end

		if (itemIndex > 0 and itemIndex <= numItems) then
			rec = list[itemIndex];

			playerTable = EkInspect_InspectList[rec.name];
			detailTable = playerTable.details[rec.index];

			color = RAID_CLASS_COLORS[playerTable.class2];
			if (not color) then
				color = { r = 1.0, g = 1.0, b = 1.0 };
			end

			fs = _G["EkInspect_DetailsFrame_Button"..i.."Name"];
			fs:SetTextColor(color.r, color.g, color.b, 1);
			fs:SetText(rec.name);
			fs:Show();

			fs = _G["EkInspect_DetailsFrame_Button"..i.."Level"];
			fs:SetTextColor(color.r, color.g, color.b, 1);
			fs:SetText(detailTable["level"]);
			fs:Show();

			fs = _G["EkInspect_DetailsFrame_Button"..i.."Item"];
			r, g, b, hex = GetItemQualityColor(detailTable["quality"]);
			fs:SetText("|c" .. hex .. detailTable["item"] .. "|r");
			fs:Show();

			-- If need scrollbar resize columns
			if ( showScrollBar ) then
				fs:SetWidth(130);
			else
				fs:SetWidth(150);
			end

			button:Show();
		else
			button:Hide();
		end
	end

	scrollBar:SetValue(scrollBar:GetValue());
end

function EkInspect_DetailsFrame_Column_SetWidth(frame, width)
	-- ----------
	-- Set width of a column.
	-- ----------
	frame:SetWidth(width);
	_G[frame:GetName().."Middle"]:SetWidth(width - 9);
end

function EkInspect_DetailsFrame_Column_Click(sortType)
	-- ----------
	-- User clicked on a column heading.
	-- ----------
	if ( sortType ) then
		if (sortType == EkInspect_DetailsFrame_LastSort) then
			if (EkInspect_DetailsFrame_Ascend) then
				EkInspect_DetailsFrame_Ascend = nil;
			else
				EkInspect_DetailsFrame_Ascend = 1;
			end
		end

		EkInspect_DetailsFrame_Sort(sortType);
		EkInspect_DetailsFrame_ListUpdate();
	end
	PlaySound("igMainMenuOptionCheckBoxOn");
end

function EkInspect_DetailsFrame_Button_OnEnter(self)
	-- ----------
	-- User moved pointer over an item.
	-- ----------
	local sel = EkInspect_DetailsFrame_Selected; -- Table of selected subscripts
	local sub = self.itemIndex; -- Subscript stored in this button

	local xp = "LEFT";
	local yp = "BOTTOM";
	local xo = 0;
	local xthis, ythis = self:GetCenter();
	local xui, yui = UIParent:GetCenter();
	if (xthis < xui) then
		xp = "RIGHT";
		if (#EkInspect_DetailsList <= EkInspect_DetailsFrame_NumToDisplay) then
			xo = 0;
		else
			xo = 32;
		end
	end
	if (ythis < yui) then
--		yp = "TOP";
		yp = "";
		if (xthis < xui) then
			xp = "RIGHT";
		else
			xp = "LEFT";
		end
	end

	GameTooltip:SetOwner(self, "ANCHOR_" .. yp .. xp, xo, 0);

	local rec = EkInspect_DetailsList[sub];
	local playerTable = EkInspect_InspectList[rec.name];
	local detailTable = playerTable.details[rec.index];
	local text, white, msg;

	white = "|c00FFFFFF";

	local r, g, b, alpha = 1, 0.82, 0, 1;
	GameTooltip:ClearLines();

	if (detailTable["link"]) then
		GameTooltip:SetHyperlink(detailTable["link"]);
	else
		GameTooltip:AddLine(white .. EkInspect_TEXT_Details_Tooltip_Name .. "|r " .. rec.name, r, g, b, alpha, 1);
		GameTooltip:AddLine(white .. EkInspect_TEXT_Details_Tooltip_Item .. "|r " .. detailTable["item"], r, g, b, alpha, 1);
		GameTooltip:AddLine(white .. EkInspect_TEXT_Details_Tooltip_Level .. "|r " .. detailTable["level"], r, g, b, alpha, 1);
	end

	GameTooltip:Show();
end
